from .api import Account, Cheques, Invoices, System, NFT, Exchanges
from ecdsa import SigningKey, Ed25519
from httpx import AsyncClient
import time
import json


class JetAPI(Account, Cheques, Invoices, System, NFT, Exchanges):
    """
    Class for working with t.me/xJetSwapBot API.

    :author: Nick [t.me/crypt_nick]
    :author: Doc. Delpy [t.me/delpydoc]

    :license: GNUv3
    """

    def __init__(self, api_key: str, private_key: str = None, network: str = 'mainnet'):
        """
        Class initialization.

        :param `api_key` [str]: API key
        :param `private_key` [str]: Private key
        :param `mainnet` [bool or `xJetNet`]: Mainnet or testnet (`True`/`xJetNet.MAINNET` for mainnet, else - `False`/`xJetNet.TESTNET`)
        """

        if network in ["mainnet", "testnet", "localnet"]:
            network = {
                "mainnet": 1,
                "testnet": 2,
                "localnet": 3
            }[network]
        
        assert network in [1,2,3], 'Invalid "network" value!'
                        
        assert type(api_key) in [str, bytes], 'Invalid "api_key" value!'
        if type(api_key) == bytes: api_key = api_key.decode('utf-8')
        
        assert type(private_key) in [str, bytes], 'Invalid "private_key" value!'
        if type(private_key) == bytes: private_key = private_key.decode('utf-8')
        
        self.api_key = api_key
        self.private_key = bytes.fromhex(private_key) if type(private_key) == str else private_key
        self.network: str = {
                        "1": 'https://xjet.app/api/v1',
                        "2": 'https://testnet.xjet.app/api/v1',
                        "3": 'https://127.0.0.1:5000/api/v1',
                    }[str(network)] 
        self.client = AsyncClient(headers={'X-API-Key': self.api_key})

    async def request(self, method: str, request_method = "POST", **kwargs):
        """ Send request to API xJet 
        
        :param `method` [str]: Method of API
        :param `*kwargs`: Other params
        """
        try:
            request = await self.client.request(
                method = request_method,
                url = self.network + "/" + method.replace('/', ''),
                **kwargs
            )
            try:
                res = request.json()
            except BaseException:
                return {'error': 'Invalid response', 'context': request.text}
            
            if res.get('error'):
                raise Exception(res['error'])
            
            return res
        except BaseException as e:
            return {'error': f"{e}"}

    def sign_message(self, message: dict = {}) -> dict:
        """ Sign json body of reques
        
        :param `message` [dict]: Message for signing"""
        message['query_id']  = (int(time.time() + 60) << 16) if 'query_id' not in message else message['query_id']
        message['signature'] = SigningKey.from_string(self.private_key, curve=Ed25519).sign(json.dumps(message).encode()).hex()
        return message



from typing import Union


class System:

    """ System class.

    Available methods:

    * `currencies` - Get list of currencies

    """

    async def currencies(self):
        """ Get list of currencies

        :return: List of currencies

        """
        return await self.request(method = 'system.currencies')


class Account:

    """ Account methods wrapper. 
    
    Available methods:

    * `me` - Get information about API application
    * `balances` - Get balances
    * `submit_deposit` - Check for deposit
    * `withdraw` - Withdraw funds
    
    """
    
    async def me(self):
        """ Get information about API application. """
        return await self.request(method = 'account.me')

    async def balance(self):
        """ Get balance """
        return await self.request(method = 'account.balances')

    async def submit_deposit(self):
        """ Check for deposit """
        return await self.request(method = 'account.submitDeposit')

    async def withdraw(self, ton_address: str, currency: str, amount: Union[int, float]):
        """ Withdraw funds 
        
        :param `ton_address` [str]: TON address
        :param `currency` [str]: Currency of withdraw
        :param `amount` Union[int, float]: Amount of withdraw
        """
        return await self.request(
            method = 'account.withdraw', 
            json = self.sign_message({
                'ton_address': ton_address, 'currency': currency, 'amount': amount
            })
        )

    async def operations(self, limit: int = 100, offset: int = 0):
        """ Get operations history """
        return await self.request(
            method = 'account.operations',
            json = {
                'limit': limit, 'offset': offset
            }
        )


class Cheques:

    """ Cheques methods wrapper. 
    
    Available methods:

    * `cheque_create` - Create cheques
    * `cheque_status` - Get cheque status
    * `cheque_list` - Get list of cheques
    * `cheque_activate` - Activate cheque
    
    """

    async def cheque_create(self, currency: str, amount: Union[int, float], expires: int = None, description: str = '', activates_count: int = 1, groups_id: list = None, personal_id: str = None, password: str = None, **kwargs):
        """ Create cheques 
        
        :param `currency` [str]: Currency of cheque
        :param `amount` Union[int, float]: Amount of cheque
        :param `expires` [int]: Expiration time of cheque
        :param `description` [str]: Description of cheque
        :param `activates_count` [int]: Number of activations
        :param `groups_id` [list]: List of groups
        :param `personal_id` [str]: Personal id
        :param `password` [str]: Password

        """
        return await self.request(
            method='cheque.create', 
            json=self.sign_message({
                'currency': currency,
                'amount': amount,
                'expires': expires,
                'description': description,
                'activates_count': activates_count,
                'groups_id': groups_id,
                'personal_id': personal_id,
                'password': password,
                **kwargs
            })
        )

    async def cheque_status(self, cheque_id: str):
        """ Get cheque status 
        
        :param `cheque_id` [str]: Cheque id
        """
        return await self.request(method = 'cheque.status', params = {'cheque_id': cheque_id})

    async def cheque_list(self):
        """ Get list of cheques """
        return await self.request(method = 'cheque.list')

    async def cheque_cancel(self, cheque_id: str):
        """ Cancel cheque 
        
        :param `cheque_id` [str]: Cheque id
        """
        return await self.request(method = 'cheque.cancel', json = {'cheque_id': cheque_id})


class Invoices:

    """ Invoice methods wrapper. 
    
    Available methods:
    
    * invoice_create()
    * invoice_status()
    * invoice_list()

    """

    async def invoice_create(self, currency: str, amount: Union[int, float], description: str = None, max_payments: int = 1, expires: int = None):
        """ Create invoice 
        
        :param `currency` [str]: Currency of invoice
        :param `amount` Union[int, float]: Amount of invoice
        :param `description` [str]: Description of invoice
        :param `max_payments` [int]: Max payments
        """
        return await self.request(
            method = 'invoice.create', 
            json = {
                'currency': currency.lower(),
                'amount': amount,
                'description': description,
                'max_payments': max_payments,
                'expires': expires
            }
        )

    async def invoice_status(self, invoice_id: str):
        """ Get invoice status
        
        :param `invoice_id` [str]: Invoice id
        """
        return await self.request(method = 'invoice.status', params = {'invoice_id': invoice_id})

    async def invoice_list(self):
        """ Get invoice list """
        return await self.request(method = 'invoice.list')


class NFT:

    """ NFT methods wrapper. 
    
    Available methods:
    
    * nft_list()
    * nft_transfer()

    """

    async def nft_list(self):
        """ List your NFTs """
        return await self.request(method = 'nft.list')

    async def nft_transfer(self, nft_address: str, to_address: str):
        """nft_transfer

        Args:
            nft_address (str): NFT Item contract address
            to_address (str): Transfer recipient address
        """
        return await self.request(
            method = 'nft.transfer',
            json = self.sign_message({
                'nft_address': nft_address,
                'to_address': to_address
            }) 
        )
        

class Exchanges:

    """ Exchange methods wrapper. 
    
    Available methods:
    
    * pairs()
    * estimate()
    * create_order()
    * order_status()

    """

    async def exchange_pairs(self):
        """ Get list of pairs """
        return await self.request(method = 'exchanges.pairs', request_method = 'GET')

    async def exchange_estimate(self, pair: list, action_type: str, amount: Union[int, float]):
        """ Estimate

        :param `pair` [list]: Pair
        :param `type` [str]: Type
        :param `amount` Union[int, float]: Amount
        
        """
        return await self.request(
            method = 'exchanges.estimate', 
            json = {
                'pair': pair, 'type': action_type, 'amount': amount
            }
        )

    async def exchange_create_order(self, pair: list, action_type: str, amount: Union[int, float], min_excepted_amount: Union[int, float]):
        """ Create order

        :param `pair` [list]: Pair
        :param `action_type` [str]: Action type
        :param `amount` Union[int, float]: Amount
        :param `min_excepted_amount` Union[int, float]: Min excepted amount
        
        """
        return await self.request(
            method = 'exchanges.createOrder', 
            json = self.sign_message({
                'pair': pair, 'type': action_type, 'amount': amount, 'min_expected_amount': min_excepted_amount
            })
        )

    async def exchange_order_status(self, order_id: str):
        """ Get order status
        
        :param `order_id` [str]: Order id
        """
        return await self.request(method = 'exchanges.orderStatus', params = {'id': order_id})



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['xjet']

package_data = \
{'': ['*']}

install_requires = \
['ecdsa>=0.18.0', 'httpx>=0.23.1']

setup_kwargs = {
    'name': 'xjet',
    'version': '0.1.5',
    'description': 'Python SDK for t.me/xJetSwapBot',
    'long_description': '# Python SDK for xJet Connect API\n\n## Authors\n- [@xJetLabs](https://github.com/xJetLabs) (forked from)\n- [@nik-1x](https://www.github.com/nik-1x)\n \n## Installation\n```shell\npip install xjet\n```\nIf requires <b>nacl</b> package, install it <i>manually</i>:\n```shell\npip install pynacl\n```\n\n## Webhook analog\n```python\nwhile True:\n    res = httpx.get(\n        \'https://api.xjet.io/v1/account.events\',\n        params={\'timeout\': 10},\n        headers={\n            \'X-API-Key\': api_key,\n        },\n        timeout=11,\n    )\n    print(res.json)\n\n    time.sleep(3)\n```\n\n\n## Usage/Examples  \n[Live example](https://replit.com/@delpydoc/xJetAPI)\n\n### Initialization\n```python\nfrom xjet import JetAPI\n\n# api_key: str\n# private_key: str\napi = JetAPI(\n    api_key="API_KEY",\n    private_key="PRIVATE_KEY",\n    network=\'mainnet\'  # mainnet / testnet\n)\n```\n\n\n### Info\n```python\nawait xjet.currencies() # Shows all saved xJetSwap currencies\n```\n\n\n### Account\n```python\nawait api.me() # get API Application information.\n# {\'id\': <str>, \'name: <str>, \'service_wallet\': <str>}\n\nawait api.balance() # get balance\n# {\n#   \'balances\': [\n#       {\'currency\': <int>, \'amount\': <float>, \n#           \'values\': {\'byn\': 0.0, \'cny\': 0.0, \'eur\': 0.0, \'gbp\': 0.0, \'kzt\': 0.0, \'rub\': 0.0, \'uah\': 0.0, \'usd\': 0.0}}], \n#   \'timestamp\': <int>\n# }\n\nawait api.submit_deposit() # check for deposit\n# {\'success\': <bool>}\n\n# ton_address: str\n# currency: str\n# amount: float\nawait api.withdraw(ton_address, currency, amount) # check for deposit\n\n# limit: int\n# offset: int\nawait xjet.operations(limit, offset) # operations\n```\n\n### Cheque\n```python\n# currency: str\n# amount: float\n# expires: [int, None]\n# description: [str, None]\n# activates_count: [int, None]\n# groups_id: [int, None]\n# personal_id: [int, None]\n# password: [str, None]\n\nawait api.cheque_create(currency, amount, expires, description, activates_count, groups_id, personal_id, password) # create cheque\n# {\'cheque_id\': <str>, \'external_link\': \'https://t.me/xJetSwapBot?start=c_<cheque_id>\'}\n\nawait api.cheque_status(cheque_id) # get cheque status\n# {\n#   \'id\': <str>, \n#   \'issuer_id\': <str>, \n#   \'amount\': <float>, \n#   \'activates_count\': <int>, \n#   \'activates: <list[str]>, \n#   \'locked_value\': <float>, \n#   \'currency\': <Str>, \n#   \'expires\': <bool>, \n#   \'description\': <str>, \n#   \'status\': \'activated/canceled/expired/active\', \n#   \'password\': <str | None>, \n#   \'groups_id\': <list[str] | None>, \n#   \'personal_id\': <int | None>, \n#   \'is_for_premium\': <bool>\n# }\n\n\nawait api.cheque_list() # get cheques on account\n# list of cheque_status\n\nawait api.cheque_cancel(cheque_id) # delete cheque\n# returns cheque_status\n```\n\n\n### Invoice\n```python\n# currency: str\n# amount: float\n# description: [str, None]\n# max_payments: [int, None]\nawait api.invoice_create(currency, amount, description, max_payments) # create invoice\n# {\'invoice_id\': <str>, \'external_link\': \'https://t.me/xJetSwapBot?start=inv_<cheque_id>\'}\n\nawait api.invoice_status(invoice_status) # get invoice status\n# {\n#   \'id\': <str>, \n#   \'description\': <str>, \n#   \'currency\': <str>, \n#   \'amount\': <float>, \n#   \'max_amount\': None, \n#   \'min_amount\': None, \n#   \'payments\': [{\'telegram_id\': <int>, \'amount\': <float>, \'comment\': [str | None}, ... ], \n#   \'max_payments\': 1, \n#   \'after_pay\': [], \n#   \'created\': \'2023-04-20T22:55:24.313000\'\n# }\n\nawait api.invoice_list() # get invoices on account\n# list of invoice_status\n```\n\n```python\n# NFT methods\nawait api.nft_list()\nawait api.nft_transfer(nft_address, to_address)\n```\n\n## License\n[GNUv3](https://github.com/nik-1x/pyxJetAPI/blob/main/LICENSE)  \n',
    'author': 'delpydoc',
    'author_email': 'delpydoc@proton.me',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://t.me/xJetSwapBot',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



import asyncio
import functools
import inspect

from . import JetAPI


def _syncify_wrap(t, method_name):
    method = getattr(t, method_name)

    @functools.wraps(method)
    def syncified(*args, **kwargs):
        coro = method(*args, **kwargs)
        loop = asyncio.get_event_loop()
        if loop.is_running():
            return coro
        else:
            return loop.run_until_complete(coro)

    # Save an accessible reference to the original method
    setattr(syncified, '__tl.sync', method)
    setattr(t, method_name, syncified)


def syncify(*types):
    """
    Converts all the methods in the given types (class definitions)
    into synchronous, which return either the coroutine or the result
    based on whether ``asyncio's`` event loop is running.
    """
    # Our asynchronous generators all are `RequestIter`, which already
    # provide a synchronous iterator variant, so we don't need to worry
    # about asyncgenfunction's here.
    for t in types:
        for name in dir(t):
            if not name.startswith('_') or name == '__call__':
                if inspect.iscoroutinefunction(getattr(t, name)):
                    _syncify_wrap(t, name)


syncify(JetAPI)

__all__ = [
    'JetAPI'
]
